##############################################
#Hidden data: The real parameters and the distributions are UNKNOWN for the researcher
##############################################
#Generate 100 random values from a Normal distribution with the mean and the sd given in the command
t1 <- rnorm(100,mean=10,sd=2) 
t2 <- rnorm(100,mean=9,sd=1)
t3 <- rnorm(100,mean=9,sd=5)

mat.t <- cbind(t1,t2,t3) #Create a matrix from the 3 vectors
colnames(mat.t) <- c("tissue1","tissue2","tissue3") #Give a name to each column

##############################################
#This is the data that the researcher obtain from experiments:
##############################################
write.table(mat.t,file="Data_tissues_100.txt",quote=FALSE,row.names=FALSE) #write a file with the data


########################################################################################################################
#EXERCISE
########################################################################################################################
# Calculate the mean differences between the tissues for (i) a sample with 100 individuals and for (ii) a sample with 10 individuals (paired data). 
# Contrast the hypothesis that each pair of tissues are equal (null hypothesis).
#TO DO:
# Plot the distributions using hist and/or density. and keep the plots into a file (pdf).
# Do a non-parametric bootstrap (sample) to estimate the mean differences under the hypothesis that each pair of tissues are equal and calculate their probability.
# Do parametric tests and calculate probabilities.
# Do parametric simulations and calculate probabilities.
########################################################################################################################



 
##############################################
#ANSWER (one way to do)
##############################################

nsamples <- 100 #Try different number of samples to see the effect on the P-value
nresamplings <- 1000 #Try more or less iterations to have better distributions

tissues <- read.table(file="Data_tissues_100.txt",header=TRUE,nrows=nsamples) #Collect data
class(tissues) 
tissues <- as.matrix(tissues) #I prefer to work with data as a matrix, but is possible that is not necessary in this example

##########Calculate the sampled mean and the sdev for each tissue:
mm <- apply(tissues,2,mean)
ss <- apply(tissues,2,sd)

########################################################################################################################
# PLOTS
########################################################################################################################
##########Let's do a PLOT using the densities:
min.value <- min(tissues)
max.value <- max(tissues)

pdf(sprintf("Plot_densities_Histograms_tissue_expression_nsamples%d.pdf",nsamples),width=10,height=10)
plot(density(tissues[,1]),col="black",xlim=c(min.value - 1/10*(max.value-min.value), max.value + 1/10*(max.value-min.value)),ylim=c(0,0.5),main=sprintf("Density distributions for:\n tissue1, tissue2 and tissue3\n means: %.3f, %.3f, %.3f",mm[1],mm[2],mm[3]),xlab="Expression values",ylab="Densities")
lines(density(tissues[,2]),col="red")
lines(density(tissues[,3]),col="blue")
legend("topleft",legend=c("tissue1","tissue2","tissue3"),text.col=c("black","red","blue"),bty="n")

##########Let's do a plot using histogrames and densities:
h1 <- hist(tissues[,1],nclass=sqrt(nsamples),plot=F) #Is common to divide the the data in a number given by the square root of the total number of samples
h2 <- hist(tissues[,2],nclass=sqrt(nsamples),plot=F)
h3 <- hist(tissues[,3],nclass=sqrt(nsamples),plot=F)
plot(h1$mids,h1$density,type="h",col="black",xlim=c(min.value - 1/10*(max.value-min.value), max.value + 1/10*(max.value-min.value)),ylim=c(0,0.5),main=sprintf("Histograms and densitiy curves for:\n tissue1, tissue2 and tissue3\n means: %.3f, %.3f, %.3f",mm[1],mm[2],mm[3]),xlab="Expression values",ylab="Frequency/class")
lines(h1$mids,h1$density,type="s",col="black")
lines(density(tissues[,1]),col="black",lwd=2)
lines(h2$mids,h2$density,type="h",col="red")
lines(h2$mids,h2$density,type="s",col="red")
lines(density(tissues[,2]),col="red",lwd=2)
lines(h3$mids,h3$density,type="h",col="blue")
lines(h3$mids,h3$density,type="s",col="blue")
lines(density(tissues[,3]),col="blue",lwd=2)
legend("topleft",legend=c("tissue1","tissue2","tissue3"),text.col=c("black","red","blue"),bty="n")
dev.off()

##########You can also do separately:
pdf(sprintf("Individual_Histograms_tissue_expression_nsamples%d.pdf",nsamples),width=23,height=8)
##########Divide the plot in several plots, indicated by the number of rows and columns
par(mfrow=c(1,3)) 
hist(tissues[,1],freq=F,nclass=sqrt(nsamples),xlim=c(min.value - 1/10*(max.value-min.value), max.value + 1/10*(max.value-min.value)),main=sprintf("Histogram for t1 with mean %.3f and sdev %.3f",mm[1],ss[1]),xlab="Expression values",ylab="Frequency/class")
hist(tissues[,2],freq=F,nclass=sqrt(nsamples),xlim=c(min.value - 1/10*(max.value-min.value), max.value + 1/10*(max.value-min.value)),main=sprintf("Histogram for t2 with mean %.3f and sdev %.3f",mm[2],ss[2]),xlab="Expression values",ylab="Frequency/class")
hist(tissues[,3],freq=F,nclass=sqrt(nsamples),xlim=c(min.value - 1/10*(max.value-min.value), max.value + 1/10*(max.value-min.value)),main=sprintf("Histogram for t3 with mean %.3f and sdev %.3f",mm[3],ss[3]),xlab="Expression values",ylab="Frequency/class")
dev.off()

########################################################################################################################
# NON-PARAMETRIC SIMULATIONS
########################################################################################################################
##########Now calculate the differences in the mean between all combinations (3 here):
ntissues <- length(tissues[1,])

##########Let's do a function to save time (later):
calculate.mean.diffs <- function(x,ncols) { #x is a matrix and ncols the number of columns to calculate the mean differences
	tissues <- x
	ntissues <- ncols
	
	obs.diff <- array(NA,dim=c(ntissues*(ntissues-1)/2)) #Let's do all combinations (n*(n-1)/2) and put into a one-dimensional vector
	k <- 1
	for(i in 1:(ntissues-1)) {
		for(j in (i+1):ntissues) { 
			obs.diff[k] <- mean(tissues[,i] -tissues[,j])
			k <- k + 1
		}
	}
	obs.diff
}

########## Call the function to have the mean of observed differences between tissues
obs.diff <- calculate.mean.diffs(x=tissues,ncols=ntissues)
obs.diff #OBSERVED RESULT

########## Let's do nresamplings replicates by simulation for all combinations (n*(n-1)/2) using the same function. 
ncombinations <- (ntissues*(ntissues-1)/2)
sim.diff <- array(NA,dim=c(nresamplings,ncombinations)) 
k <- 1
for(i in 1:(ntissues-1)) {
	for(j in (i+1):ntissues) { 
		for(r in 1:nresamplings) {
			#simulation <- matrix(sample(tissues[,c(i,j)],size=2*nsamples,replace=TRUE),nrow=nsamples,ncol=2) #if tissues have equal mean (null hypothesis) we can resample all data between tissues (UNPAIRED DATA)
			simulation <- t(apply(tissues[,c(i,j)],1,sample,size=2,replace=TRUE)) # resample the data from each individual (PAIRED DATA)
			sim.diff[r,k] <- calculate.mean.diffs(x=simulation,ncols=2) #calculate the difference between mixed data (should be close to zero)
		}
		k <- k + 1
	}
}
sim.diff #SIMULATED RESULTS:

##########Let's see the distribution of the simulated values and the observed value for each comparison:
pdf(sprintf("Observed_and_non_parametric_simulated_differences_between_tissues_nsample%d.pdf",nsamples),width=24,height=8)
par(mfrow=c(1,ncombinations)) 
Pvalue <- array(NA,dim=c(ncombinations))
Pvalue.density <- Pvalue

k <- 1
for(i in 1:(ntissues-1)) {
	for(j in (i+1):ntissues) { 
		########## Calculate the probability to have the observed result given the null hypothesis (equal mean) from real data AND from the expected density ###############
		Pvalue[k] <- 0.5 - abs(0.5-sum(obs.diff[k] <= sim.diff[,k])/nresamplings) + 0.5 - abs(0.5-sum(obs.diff[k] > sim.diff[,k])/nresamplings) + sum(obs.diff[k] == sim.diff[,k])/nresamplings #for both tails
		dens <- density(sim.diff[,k])
		Pvalue.density[k] <- 0.5 - abs(0.5-sum(dens$y[obs.diff[k] <= dens$x])/sum(dens$y)) + 0.5 - abs(0.5-sum(dens$y[obs.diff[k] > dens$x])/sum(dens$y)) + sum(dens$y[obs.diff[k] == dens$x])/sum(dens$y) #for both tails
		########## BUT, we did several tests!, we should correct for multiple testing 
		########## (that is, if you do 100 tests and you have 5% type 1 error, you expect that 5 tests will be significant by chance. 
		########## To correct it by using the simplest (but the most conservative) method: Bonferroni (divide the type 1 error by the number of tests you did). 
		########## Look at False Discovery Test algorithms or others too)
		########## Do the plot
		min.s.value <- min(sim.diff[,k],obs.diff[k])
		max.s.value <- max(sim.diff[,k],obs.diff[k])
		oh <- hist(sim.diff[,k],nclass=sqrt(nresamplings),plot=F)
		max.y.value <- max(oh$density,density(sim.diff[,k])$y)
		plot(oh$mids,oh$density,type="h",col="black",xlim=c(min.s.value - 1/10*(max.s.value-min.s.value), max.s.value + 1/10*(max.s.value-min.s.value)),ylim=c(0, max.y.value),main=sprintf("Histograms and densitiy curves for \n the mean differences between sampled tissues %.0f and %.0f\n Simulated mean: %.3f Observed: %.3f",i,j,mean(sim.diff[,k]),obs.diff[k]),xlab="Differences",ylab="Density")
		lines(oh$mids,oh$density,type="s",col="black")
		lines(density(sim.diff[,k]),col="black")
		abline(v= obs.diff[k],col="magenta",lwd=2)
		abline(h= 0,col="black")
		legend("topleft",legend=c("Simulated mean differences","Observed mean difference"),text.col=c("black","magenta"),bty="n")
		text(x=obs.diff[k],y=0,labels=sprintf("P-value=%.3f \n P-value_density=%.3f",Pvalue[k],Pvalue.density[k]),col="magenta")
		k <- k + 1
	}
}

dev.off()

########################################################################################################################
# PARAMETRIC TEST
########################################################################################################################

#If we are interested in doing a PARAMETRIC TESTS:
#Perhaps look at the data. Check if the data fit to a Normal distribution (for each column).
#In case the data fit to Normal distribution, estimate the mean and the sd (the parameters of the model).
#Then check the differences between models using the features of the Normal distribution.

#First do a shapiro.test to check Normality and plot the quantiles to see the divergence of from a Normal
pdf(file=sprintf("Normality_tests_nsamples%.0f.pdf",nsamples),width=10,height=10)
for(i in 1:dim(tissues)[2]) {
	result.test <- shapiro.test(tissues[,i])
	qqnorm(tissues[,i],main=sprintf("Normal QQ-plot for tissue %.0f\n Probability to be into a Normal is %.3f",i,result.test$p))
	qqline(tissues[,i],col="red")
}
dev.off()

#Simple estimation of real mean and sd
mm <- apply(tissues,2,mean)
ss <- apply(tissues,2,sd)

ntissues <- length(tissues[1,])
ncombinations <- (ntissues*(ntissues-1)/2)
k <- 1
for(i in 1:(ntissues-1)) {
	for(j in (i+1):ntissues) { 
		result.t.test <- t.test(tissues[,i],tissues[,j],paired=TRUE,alternative="two.sided",mu=0,var.equal=FALSE,conf.level = (1-0.05/ncombinations)) #Note: check the class for result.t.test to know the values to keep. Here we use Bonferroni for correcting the confidence interval.
		text.result <- sprintf("Two-sample T-test for unequal variances and paired data. Contrasting the null Hypothesis Diff=0: tissue %.0f vs %.0f: t = %.3f, df = %.3f, p-value=%.3e. Mean estimates are %.3f and %.3f, respectively. Confidence interval = %.3f (Bonferroni correction for multiple tests).",i,j, result.t.test$statistic, result.t.test$parameter, result.t.test$p.value,result.t.test$estimate[1],result.t.test$estimate[2],(1-0.05/ncombinations))
		if(k==1) {napp<-FALSE
		} else {napp<-TRUE}
		write.table(x=text.result,file=sprintf("Differences_between_tissues_from_Ttest_nsamples%.0f.txt",nsamples),append= napp,quote=FALSE,row.names=FALSE,col.names=FALSE)
		k <- k + 1
	}
}

########################################################################################################################
# PARAMETRIC SIMULATION
########################################################################################################################
# If we want to do a PARAMETRIC SIMULATION, we should estimate the parameters for each tissue and then simulate the data using the estimated parameters:
# Then, we can calculate the confidence intervals for each of the difference distributions and check if the value "0" (no differences) is inside the distribution and with what probability:
# In this case, we assume a Normal distribution for each case (as we saw before) we would do:
nreplicates <- 1000 #numer of distributions that we will simulate

#Simple estimation of real mean and sd
mm <- apply(tissues,2,mean)
ss <- apply(tissues,2,sd)

ntissues <- length(tissues[1,])
ncombinations <- (ntissues*(ntissues-1)/2)

tissue.sim <- array(NA,dim=c(nsamples,ntissues,nreplicates))
for(r in 1:nreplicates) { # we do many simulated distributons
	for(i in 1: ntissues) {
		tissue.sim[,i,r] <- rnorm(nsamples,mean=mm[i],sd=ss[i])
	}
}

####### Then, we obtain the confidence intervals of the differences from the simulated distributions
####### Here seems unnecessary because we have a Normal distribution, but if we had a different distribution (for example an exponential distribution) we can test differences by simulation.

########## Recover the previous function to calculate differences. That is: calculate.mean.diffs 

##########Call the function to have the mean of observed differences between tissues
obs.diff <- calculate.mean.diffs(x=tissues,ncols=ntissues)
obs.diff #OBSERVED RESULT

##########
########## NON-PARAMETRIC SIMULATIONS
##########
sim.par.diff <- array(NA,dim=c(nreplicates,ncombinations)) 
k <- 1
for(i in 1:(ntissues-1)) {
	for(j in (i+1):ntissues) { 
		for(r in 1: nreplicates) {
			sim.par.diff[r,k] <- calculate.mean.diffs(x= tissue.sim[,c(i,j),r],ncols=2) #calculate the difference between simulated distributions
		}
		k <- k + 1
	}
}
sim.par.diff #SIMULATED RESULTS:

######### Now we can calculate the confidence intervals for each of the difference distributions and check if the value "0" (no differences) is inside the distribution and with what probability:

##########Let's see the distribution of the simulated values and the observed value for each comparison:
pdf(sprintf("Parametric_simulated_differences_between_tissues_nsample%d.pdf",nsamples),width=24,height=8)
par(mfrow=c(1,ncombinations)) 
Pvalue <- array(NA,dim=c(ncombinations))
Pvalue.density <- Pvalue

k <- 1
for(i in 1:(ntissues-1)) {
	for(j in (i+1):ntissues) { 
		########## Calculate the probability to have the observed result given the null hypothesis (equal mean) from real data AND from the expected density ###############
		Pvalue[k] <- 0.5 - abs(0.5-sum(0 <= sim.par.diff[,k])/nreplicates) + 0.5 - abs(0.5-sum(0 > sim.par.diff[,k])/nreplicates) + sum(0 == sim.par.diff[,k])/nreplicates #for BOTH tails
		dens <- density(sim.par.diff[,k])
		Pvalue.density[k] <- 0.5 - abs(0.5-sum(dens$y[0 <= dens$x])/sum(dens$y)) + 0.5 - abs(0.5-sum(dens$y[0 > dens$x])/sum(dens$y)) + sum(dens$y[0 == dens$x])/sum(dens$y) #for BOTH tails
		########## Do the plot
		min.s.value <- min(sim.par.diff[,k],0)
		max.s.value <- max(sim.par.diff[,k],0)
		oh <- hist(sim.par.diff[,k],nclass=sqrt(nreplicates),plot=F)
		max.y.value <- max(oh$density,density(sim.par.diff[,k])$y)
		plot(oh$mids,oh$density,type="h",col="black",xlim=c(min.s.value - 1/10*(max.s.value-min.s.value), max.s.value + 1/10*(max.s.value-min.s.value)),ylim=c(0, max.y.value),main=sprintf("Histograms and densitiy curves for \n the mean differences between sampled tissues %.0f and %.0f\n Simulated mean: %.3f",i,j,mean(sim.par.diff[,k])),xlab="Differences",ylab="Density")
		lines(oh$mids,oh$density,type="s",col="black")
		lines(density(sim.par.diff[,k]),col="black")
		abline(v= 0,col="magenta",lwd=2)
		abline(h= 0,col="black")
		legend("topleft",legend=c("Simulated mean differences","No differences"),text.col=c("black","magenta"),bty="n")
		text(x=0,y=0,labels=sprintf("P-value=%.5f \n P-value_density=%.5f",Pvalue[k],Pvalue.density[k]),col="magenta")
		k <- k + 1
	}
}

dev.off()

